@@ -1,6 +1,7 @@
 from django.contrib import admin
+from django import forms
 
-from .models import Rotation, Entry
+from .models import Rotation, Entry, EntryCharacter
 
 
 @admin.register(Rotation)
@@ -11,15 +12,15 @@ class RotationAdmin(admin.ModelAdmin):
 
 
 class EntryCharacterInline(admin.TabularInline):
-    model = Entry.shares.through
-    raw_id_fields = ('user', )
+    model = EntryCharacter
+    raw_id_fields = ('user', 'user_character',)
+    readonly_fields = ('role',)
 
 
 @admin.register(Entry)
 class EntryAdmin(admin.ModelAdmin):
     raw_id_fields = ('rotation', )
-    readonly_fields = ('created_by', )
+    readonly_fields = ('created_by', 'created_at', 'updated_at',)
     inlines = (EntryCharacterInline, )
     search_fields = ('id', )
     list_display = ('pk', 'rotation', 'estimated_total', 'created_by', 'created_at', )
-    exclude = ('total_shares_count', )
