@@ -4,6 +4,7 @@ from django.utils import timezone
 from django.db.models.functions import Coalesce
 
 from allianceauth.services.hooks import get_extension_logger
+from allianceauth.eveonline.models import EveCharacter
 
 logger = get_extension_logger(__name__)
 
@@ -82,28 +83,22 @@ class Rotation(models.Model):
 
 
 class EntryCharacter(models.Model):
-    share_count = models.PositiveIntegerField(default=1)
     entry = models.ForeignKey('Entry', on_delete=models.CASCADE, related_name='ratting_shares')
     user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='ratting_shares')
+    user_character = models.ForeignKey(EveCharacter, on_delete=models.CASCADE, related_name='ratting_shares')
+    role = models.ForeignKey('EntryRole', on_delete=models.RESTRICT, related_name='shares')
+
+    site_count = models.PositiveIntegerField(default=1)
     helped_setup = models.BooleanField(default=False)
     estimated_share_total = models.FloatField(default=0)
     actual_share_total = models.FloatField(default=0)
 
-    @property
-    def estimated_total(self):
-        return (self.entry.estimated_total_after_tax / self.entry.total_shares_count) * self.share_count
-
-    @property
-    def actual_total(self):
-        return (self.entry.actual_total_after_tax / self.entry.total_shares_count) * self.share_count
-
     class Meta:
         default_permissions = ()
 
 
 class Entry(models.Model):
-    rotation = models.ForeignKey('Rotation', on_delete=models.CASCADE, related_name='entries')
-    shares = models.ManyToManyField(settings.AUTH_USER_MODEL, through=EntryCharacter, related_name='+')
+    rotation = models.ForeignKey(Rotation, on_delete=models.CASCADE, related_name='entries')
     estimated_total = models.FloatField(default=0)
 
     created_at = models.DateTimeField(auto_now_add=True)
@@ -116,7 +111,7 @@ class Entry(models.Model):
 
     @property
     def total_shares_count(self):
-        return self.ratting_shares.aggregate(val=Coalesce(models.Sum('share_count'), 0))["val"]
+        return self.ratting_shares.aggregate(val=models.Count('user_id', distinct=True))['val']
 
     @property
     def estimated_total_after_tax(self):
@@ -128,19 +123,64 @@ class Entry(models.Model):
         return self.estimated_total_after_tax * self.rotation.sales_percentage
 
     def update_share_totals(self):
-        shares_count = self.total_shares_count
-        if shares_count == 0:
+        sum_sites = self.ratting_shares.aggregate(val=Coalesce(models.Sum('site_count'), 0))['val']
+
+        self.roles.alias(char_count=models.Count('shares')).filter(char_count=0).delete()
+        num_roles = self.roles.count()
+        if sum_sites == 0 or num_roles == 0:
             self.delete()
         else:
             self.save()
 
-            self.ratting_shares\
-                .annotate(estimated_total_after_tax=models.Value(self.estimated_total) * (models.Value(100) - models.Value(self.rotation.tax_rate)) / models.Value(100))\
-                .annotate(actual_total_after_tax=models.F('estimated_total_after_tax') * models.Value(self.rotation.sales_percentage))\
-                .update(
-                    estimated_share_total=(models.F('estimated_total_after_tax') / models.Value(shares_count)) * models.F('share_count'),
-                    actual_share_total=(models.F('actual_total_after_tax') / models.Value(shares_count)) * models.F('share_count')
-                )
+            estimated_total_after_tax = self.estimated_total * (100 - self.rotation.tax_rate) / 100
+            actual_total_after_tax = estimated_total_after_tax * self.rotation.sales_percentage
+
+            if settings.DATABASES[self.ratting_shares.db]['ENGINE'] == 'django.db.backends.mysql':
+                total_value = 0
+                for role in self.roles.all():
+                    total_value += role.shares.annotate(weighted_share_value=models.F('site_count') * models.Value(role.value))\
+                        .aggregate(val=models.Sum('weighted_share_value'))['val']
+
+                for role in self.roles.all():
+                    role.shares\
+                        .annotate(weighted_share_value=models.F('site_count') * models.Value(role.value))\
+                        .annotate(relative_value=models.F('weighted_share_value') / models.Value(total_value, output_field=models.FloatField()))\
+                        .update(
+                            estimated_share_total=models.Value(estimated_total_after_tax) * models.F('relative_value'),
+                            actual_share_total=models.Value(actual_total_after_tax) * models.F('relative_value'),
+                        )
+            else:
+                role_val_query = EntryRole.objects.filter(pk=models.OuterRef('role_id')).values('value')
+
+                annotated_shares = self.ratting_shares\
+                    .annotate(role_val=models.Subquery(role_val_query))\
+                    .annotate(weighted_share_value=models.F('site_count') * models.F('role_val'))
+
+                total_value = annotated_shares.aggregate(val=models.Sum('weighted_share_value'))['val']
+
+                annotated_shares\
+                    .annotate(relative_value=models.F('weighted_share_value') / models.Value(total_value, output_field=models.FloatField()))\
+                    .update(
+                        estimated_share_total=models.Value(estimated_total_after_tax) * models.F('relative_value'),
+                        actual_share_total=models.Value(actual_total_after_tax) * models.F('relative_value'),
+                    )
+
+    class Meta:
+        default_permissions = ()
+
+
+class EntryRole(models.Model):
+    entry = models.ForeignKey(Entry, on_delete=models.CASCADE, related_name='roles')
+
+    name = models.CharField(max_length=64)
+    value = models.PositiveIntegerField('relative role value', help_text="Relative role value. Share values are computed using this field. If there are 2 roles with 10 and 15, they'll receive 10/25 and 15/25 of the share value.")
+
+    def __str__(self) -> str:
+        return self.name
+
+    @property
+    def approximate_percentage(self):
+        return self.value * 100 / self.entry.roles.aggregate(val=models.Sum('value'))['val']
 
     class Meta:
         default_permissions = ()
@@ -154,6 +194,6 @@ class RotationSetupSummary(models.Model):
     valid_setups = models.IntegerField()
 
     class Meta:
-        managed = False  # this is a view, check 0003
+        managed = False  # this is a view, check 0003 and 0005
         db_table = 'allianceauth_pve_setup_summary'
         default_permissions = ()
